#include "CommonShader.h"


CommonShader::CommonShader(ID3D11Device* pd3dDevice){

	m_pd3dDevice = pd3dDevice;



}


CommonShader::~CommonShader(){


}

COM_SHADER_LIST* CommonShader::FindList(TCHAR* szFileName, LPCSTR szEntryPoint, LPCSTR szShaderModel){

	for(COM_SHADER_LIST* p = this->first; p ; p = p->next){
		if(_tcscmp(p->szFileName, szFileName)==0){
			if(strcmp(p->szEntryPoint, szEntryPoint)==0){
				if(strcmp(p->szShaderModel, szShaderModel)==0){
					return p;
				}
			}
		}
	}

	return NULL;
}

COM_SHADER_LIST* CommonShader::FindList(LPVOID pShader){

	for(COM_SHADER_LIST* p = this->first; p ; p = p->next){
		if((LPVOID)(p->pVertexShader) == pShader){
			return p;
		}
	}

	return NULL;
}



HRESULT CommonShader::CompileShaderFromFile( TCHAR* szFileName, LPCSTR szEntryPoint, LPCSTR szShaderModel, ID3DBlob** ppBlobOut ){
    HRESULT hr = S_OK;

    DWORD dwShaderFlags = D3DCOMPILE_ENABLE_STRICTNESS;
#if defined( DEBUG ) || defined( _DEBUG )
    // Set the D3DCOMPILE_DEBUG flag to embed debug information in the shaders.
    // Setting this flag improves the shader debugging experience, but still allows 
    // the shaders to be optimized and to run exactly the way they will run in 
    // the release configuration of this program.
    dwShaderFlags |= D3DCOMPILE_DEBUG;
#endif

    ID3DBlob* pErrorBlob;
	
    hr = D3DX11CompileFromFile( szFileName, NULL, NULL, szEntryPoint, szShaderModel, 
        dwShaderFlags, 0, NULL, ppBlobOut, &pErrorBlob, NULL );
/*		
	FILE* fp = _tfopen(szFileName, _T("rb"));
	fseek(fp, 0, SEEK_END);
	long filesz = ftell(fp);
	char* srcData = new char[filesz+1];
	fseek(fp, 0, SEEK_SET);
	fread(srcData, filesz, 0, fp);
	fclose(fp);

	hr = D3DCompile(srcData,filesz, NULL, NULL, NULL, szEntryPoint, szShaderModel, 
        dwShaderFlags, 0, ppBlobOut, &pErrorBlob );
*/
	if( FAILED(hr) )
    {
        if( pErrorBlob != NULL )
            OutputDebugStringA( (char*)pErrorBlob->GetBufferPointer() );
        if( pErrorBlob ) pErrorBlob->Release();
        return hr;
    }
    if( pErrorBlob ) pErrorBlob->Release();

    return S_OK;
}

ID3D11VertexShader* CommonShader::LoadVertexShader( TCHAR* szFileName, LPCSTR szEntryPoint, LPCSTR szShaderModel, 
								const D3D11_INPUT_ELEMENT_DESC* inputElements, UINT numElements, ID3D11InputLayout** ppInputLayout){

	COM_SHADER_LIST* p = FindList(szFileName,szEntryPoint, szShaderModel);
	if(p){
		p->refCount++;
		*ppInputLayout = p->pInputLayout;
		return p->pVertexShader;
	}

	// Compile the vertex shader
    ID3DBlob* pVSBlob = NULL;
    HRESULT hr = CompileShaderFromFile( szFileName, szEntryPoint, szShaderModel, &pVSBlob );
    if( FAILED( hr ) ) {
        return NULL;
    }

	// Create the vertex shader
	ID3D11VertexShader* pVertexShader;
	hr = m_pd3dDevice->CreateVertexShader( pVSBlob->GetBufferPointer(), pVSBlob->GetBufferSize(), NULL, &pVertexShader );
	if( FAILED( hr ) ) {	
		pVSBlob->Release();
        return NULL;
	}


    // Create the input layout
	ID3D11InputLayout* pInputLayout;
	hr = m_pd3dDevice->CreateInputLayout( inputElements, numElements, pVSBlob->GetBufferPointer(),
                                          pVSBlob->GetBufferSize(), &pInputLayout );
	pVSBlob->Release();
	if( FAILED( hr ) ){
		pVertexShader->Release();
        return NULL;
	}

	p = AddFirst();
	p->pVertexShader = pVertexShader;
	p->pInputLayout = pInputLayout;
	p->type = TYPE_VERTEXSHADER;
	p->refCount = 1;
	p->szFileName = _tcsdup(szFileName);
	p->szEntryPoint = _strdup(szEntryPoint);
	p->szShaderModel = _strdup(szShaderModel);
	
	
	*ppInputLayout = pInputLayout;
    return pVertexShader;
}

ID3D11GeometryShader* CommonShader::LoadGeometryShader( WCHAR* szFileName, LPCSTR szEntryPoint, LPCSTR szShaderModel){
	
	COM_SHADER_LIST* p = FindList(szFileName,szEntryPoint, szShaderModel);
	if(p){
		p->refCount++;
		return p->pGeometryShader;
	}

	// Compile the pixel shader
	ID3DBlob* pPSBlob = NULL;
    HRESULT hr = CompileShaderFromFile( szFileName, szEntryPoint, szShaderModel, &pPSBlob );
	if( FAILED( hr ) ){
		return NULL;
    }

	// Create the Geometry shader
	ID3D11GeometryShader* pGeometryShader;
	hr = m_pd3dDevice->CreateGeometryShader( pPSBlob->GetBufferPointer(), pPSBlob->GetBufferSize(), NULL, &pGeometryShader );
	pPSBlob->Release();
	if( FAILED( hr ) ){
        return NULL;
	}

	p = AddFirst();
	p->pGeometryShader = pGeometryShader;
	p->type = TYPE_GEOMETRYSHADER;
	p->refCount = 1;
	p->szFileName = _tcsdup(szFileName);
	p->szEntryPoint = _strdup(szEntryPoint);
	p->szShaderModel = _strdup(szShaderModel);


    return pGeometryShader;
}

ID3D11PixelShader* CommonShader::LoadPixelShader( WCHAR* szFileName, LPCSTR szEntryPoint, LPCSTR szShaderModel){
	
	COM_SHADER_LIST* p = FindList(szFileName,szEntryPoint, szShaderModel);
	if(p){
		p->refCount++;
		return p->pPixelShader;
	}

	// Compile the pixel shader
	ID3DBlob* pPSBlob = NULL;
    HRESULT hr = CompileShaderFromFile( szFileName, szEntryPoint, szShaderModel, &pPSBlob );
	if( FAILED( hr ) ){
		return NULL;
    }

	// Create the pixel shader
	ID3D11PixelShader* pPixelShader;
	hr = m_pd3dDevice->CreatePixelShader( pPSBlob->GetBufferPointer(), pPSBlob->GetBufferSize(), NULL, &pPixelShader );
	pPSBlob->Release();
	if( FAILED( hr ) ){
        return NULL;
	}

	p = AddFirst();
	p->pPixelShader = pPixelShader;
	p->type = TYPE_PIXELSHADER;
	p->refCount = 1;
	p->szFileName = _tcsdup(szFileName);
	p->szEntryPoint = _strdup(szEntryPoint);
	p->szShaderModel = _strdup(szShaderModel);


    return pPixelShader;
}

void CommonShader::UnloadShader(LPVOID pShader){


	COM_SHADER_LIST* p = FindList(pShader);
	if(p){
		p->refCount--;
		if(p->refCount == 0){
			Delete(p);
		}
	}

}