
#define _CRT_SECURE_NO_DEPRECATE

#include <stdlib.h>
#include <string.h>
#include "md3_reader.h"

static const int LINE_LEN = 2048;


static const int SPECIES_COUNT = 118 + 1;

static const int ERROR_MISSING_PARTICLE = -3;
static const int ERROR_OVERFLOW_PARTICLE = -4;


//OpenMXœ͂錳fL//
static const char* openmx_species_list[] = {
		"E",		//emptyO//
		"H", "He",	//1
		"Li", "Be",	"B", "C", "N", "O", "F", "Ne",		//2
		"Na", "Mg",	"Al", "Si", "P", "S", "Cl", "Ar",	//3
		"K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn", "Ga", "Ge", "As", "Se", "Br", "Kr",	//4
		"Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", "In", "Sn", "Sb", "Te", "I", "Xe",	//5
		"Cs", "Ba",	//6(Cs, Ba)
		"La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb", "Lu",	////6(^mCh)
		"Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg", "Tl", "Pb", "Bi", "Po", "At", "Rn",//6(Hf-Rn)
		"Fr", "Ra",	//7(Fr, Ra)
		"Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm", "Md", "No", "Lr",	////7(AN`mCh)
		//104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,//7(Rf-Uuo)
};


//fL̕񂩂猴qԍԂ//
inline static int GetAtomicNumber(const char* atomicsymbol){
	for(int i = 0; i < SPECIES_COUNT; i++){
		if(strcmp(atomicsymbol, openmx_species_list[i])==0){
			return i;
		}
	}
	return -1;
}


/*
static int GetAtomicNumber(const char* atomicsymbol){
	
	const char symbols[] = "H\0\0""He\0" 
		"Li\0Be\0B\0\0C\0\0N\0\0O\0\0F\0\0Ne\0"
		"Na\0Mg\0Al\0Si\0P\0\0S\0\0Cl\0Ar\0"
		"K\0\0Ka\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Kr\0"
		"Rb\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xe\0"
		"Cs\0Xx\0"
		"Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0"
		"Hf\0Ta\0W\0\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0Xx\0";

	for(int i = 0; i < 86; i++){
		if(strcmp(symbols + i * 3, atomicsymbol) == 0){
			return i + 1; 
		}
	}

	return 0;
}
*/

MD3_Reader::MD3_Reader() : 
		m_fp (NULL),
		m_pcnt(-1)
{
	
	
}

MD3_Reader::~MD3_Reader(){
	Close();
}


int MD3_Reader::Open(const char* filepath){
	
	if(m_fp){
		return -2;
	}

	m_fp = fopen(filepath, "r");
	if( ! m_fp){
		m_fp = NULL;
		return -1;
	}
	
	
	//wb_ǂݍ

	char line[LINE_LEN];
	if(fgets(line,LINE_LEN,m_fp) != NULL){
		sscanf(line,"%d", &m_pcnt);
		if(m_pcnt<=0){return -3;}
		
	}
	
	

	return 0;
}

void MD3_Reader::Close(){
	if(m_fp){
		fclose(m_fp);
		m_fp = NULL;
	}
	
	m_pcnt = 0;


}


int MD3_Reader::GetParticleCount(){
	
	return m_pcnt;
	
}




int MD3_Reader::Read(int* knd, double* r, double* v, double* f, double *boxaxis, char* text){

	char line[LINE_LEN];

	//Rgs̓ǂݏo//
	if(fgets(line,LINE_LEN,m_fp) == NULL){
		m_pcnt = -2;
		return -1;
	}

	if (text){
		strcpy(text, line);
	}

	
	double *pr = r;
	double *pv = v;
	double *pf = f;
	int count = 0;
	int next_count = -2;

	while(fgets(line,LINE_LEN,m_fp) != NULL){
			
		//ŏ̃JǂݍŔ//
		//BOX or fL or l(̃t[̃wb_)//

		char* tp = strtok(line, " \t\r\n");
		if(strcmp(tp, "BOX")==0){
			for(double* pb = boxaxis; pb != boxaxis + 9; pb++){
				tp = strtok(NULL, " \t\r\n");
				*pb = strtod(tp, NULL);
			}

		}else if( ('0' <= *tp) && (*tp <= '9')){
			//̍s̃wb_ǂݍ񂾂̂ŃXgbv
			next_count = atoi(tp);
			break;

		}else{	//read atom//

			if(count == m_pcnt){	//qI[o[t[//
				next_count = ERROR_OVERFLOW_PARTICLE;
				break;
			}


			*knd = GetAtomicNumber(tp);
			knd++;

			for(double* p_end = pr + 3; pr != p_end; pr++){
				tp = strtok(NULL, " \t\r\n");
				*pr = strtod(tp, NULL);
			}

			if(pv){ //velocityNULL//
				tp = strtok(NULL, " \t\r\n");
				*pv = strtod(tp, NULL);
				pv++;
				tp = strtok(NULL, " \t\r\n");
				*pv = strtod(tp, NULL);
				pv++;
				tp = strtok(NULL, " \t\r\n");
				*pv = strtod(tp, NULL);
				pv++;
			}

			if(pf){ //forceNULL//
				tp = strtok(NULL, " \t\r\n");
				*pf = strtod(tp, NULL);
				pf++;
				tp = strtok(NULL, " \t\r\n");
				*pf = strtod(tp, NULL);
				pf++;
				tp = strtok(NULL, " \t\r\n");
				*pf = strtod(tp, NULL);
				pf++;
			}

			count++;

		}

	
	}


		if(count < m_pcnt){	//error: qȂ//
			m_pcnt = ERROR_MISSING_PARTICLE;
//		}else if(count > m_pcnt){
//			m_pcnt = ERROR_OVERFLOW_PARTICLE;
		}else{
			m_pcnt = next_count;
		}
	//̎_Ŏ̍s̃wb_[ǂ܂ɔꍇ//
	//m_pcnt = -1ɂȂĂ͂//

	return count;
}
