#ifndef __rebojf_h__
#define __rebojf_h__


#include "vec3.h"
#include "mat33.h"
#include "mod.h"
#include "atomarray.h"
#include "celldiv.h"
#include "debugprint.h"


#define		BOND_CNT	12


struct REBO_PARTICLE{
	double fc[BOND_CNT];
	double dfc[BOND_CNT];
	int NbPrt[BOND_CNT];
	int NbCnt;
	union {
		struct{
			double Nc;
			double Nh;
		};
		double Nx[2];
	};
	double dr[BOND_CNT];
	double idr[BOND_CNT];
	vec3d e[BOND_CNT];
	vec3d fij[BOND_CNT];
};


struct REBO_BONDING{
	int i;
	int j;
	int bondnum_i;
	int bondnum_j;
};


class REBO : public CellDiv {
private:
		int m_ijmax;
	
		int m_bondCnt;
		double m_Dmax[3];
		double m_Dmin[3];
		double m_Dmax2[3];
		double m_Ddiff[3];
		
		REBO_PARTICLE* m_rp;
		

		REBO_BONDING* m_bonding;

		double* m_Jij;
		
		vec3d m_Fr_v; //F x r (diagonal)
		mat33d m_Fr_m; //F x r


/////spline
#define SP_PMAXX	3
#define SP_PMAXY    3
		
		double g_spP[SP_PMAXX+1][SP_PMAXY+1][2];
		

#define SP_FMAXX	3
#define	SP_FMAXY	3
#define SP_FMAXZ	5

		double g_spF[SP_FMAXX+1][SP_FMAXY+1][SP_FMAXZ+1][3];
		double g_spFdx[SP_FMAXX+1][SP_FMAXY+1][SP_FMAXZ+1];
		double g_spFdy[SP_FMAXX+1][SP_FMAXY+1][SP_FMAXZ+1];
		double g_spFdz[SP_FMAXX+1][SP_FMAXY+1][SP_FMAXZ+1];
		
#define SP_TMAXX	3
#define SP_TMAXY	3
#define SP_TMAXZ	5
		
		double g_spT[SP_TMAXX+1][SP_TMAXY+1][SP_TMAXZ+1];
		




	void InitSpline();
	//void GetRij(int i,int j, double* dx, double* dy, double* dz);

	inline void CalcCos3_4(int i, int j, vec3d e_ij, double idr_ij, double* cosI, vec3d* cosI_dIK, vec3d* cosI_dIJ);
	inline void CalcFor3Body(int i, int j, double* cosI, double* Gsum, double* Gary, double* Gdifary, double* Gsum2);

	inline void subG(double cosVal, int mode, double *retVal);
	inline void subGdif(double cosVal, int mode, double *retVal);

	inline double CalcG(double CosVal);
	inline double CalcGdif(double CosVal);
	inline void CalcQ(double Mt_i, double *Q_i, double *Qd_i);
	
	inline double SplineP_2(double x, double y, int PartnerKind);
	
	inline void CalcForConjugation(int i, int j, double* retSsum, double* S_ki, double* dS_ki);

	inline double SplineF_2(double x, double y, double z, int PairKind);

	inline void CalcFor4Body(int i, int j, double* cosI, double* cosJ, double* cosDH2sum, double* cosDH);
	inline double SplineT_2(double x, double y, double z);
	inline void Force3Body_2(int i, int j, double V_A, double Bij, vec3d* cosI_dIK, vec3d* cosI_dIJ, double* Gary, double* Gdifary, double Gsum2, int iprt);
	inline void ForceSplineP(int i, int j, double V_A, double Bij, double Nh_i, double Nc_i);
	inline double SplinePdx_2(double x, double y, int PartnerKind);
	inline double SplinePdy_2(double x, double y, int PartnerKind);
	inline void ForcePiBond(int i, int j,double V_A,double Nt_ij,double Nt_ji,double Nconj,double Ssum_i,double *S_ki,double *dS_ki,double Ssum_j,double *S_lj,double *dS_lj);
	inline double SplineFdx_2(double x, double y, double z, int PairKind);
	inline double SplineFdz_2(double x, double y, double z, int PairKind);
	inline void Force4Body_2(int i,int j, double V_A, double T_ij, double* cosI,vec3d* cosI_dIK,vec3d* cosI_dIJ,double* cosJ,vec3d* cosJ_dJL,vec3d* cosJ_dJI,double* cosDH, int ijprt);
	inline void ForceSplineT(int i, int j, double V_A, double Nt_ij, double Nt_ji, double Nconj, double cosDH2sum, double Ssum_i, double *S_ki, double *dS_ki, double Ssum_j, double *S_lj, double *dS_lj);
	inline double SplineTdx_2(double x, double y, double z);
	inline double SplineTdz_2(double x, double y, double z);

	inline void CalcF();
	inline void CalcFJ();
	inline void CalcFJFrv();
	inline void CalcFJFrm();
	
	inline void Reset(int count);

//	int AddBondByCutOff(int i, int j, int ki, int Lidx, double xx, double yy, double zz);
	inline int AddBondByCutOff(int i, int j, int ki, int kj, int Lidx, vec3d& dd);
	/*
	inline void GetRij_periodicX(int i,int j, double* dx, double* dy, double* dz);
	inline void GetRij_periodicY(int i,int j, double* dx, double* dy, double* dz);
	inline void GetRij_periodicZ(int i,int j, double* dx, double* dy, double* dz);
	inline void GetRij_periodicXY(int i,int j, double* dx, double* dy, double* dz);
	inline void GetRij_periodicYZ(int i,int j, double* dx, double* dy, double* dz);
	inline void GetRij_periodicZX(int i,int j, double* dx, double* dy, double* dz);
	*/

	int* m_knd;
	vec3d* m_r;
	vec3d* m_p;
	vec3d* m_f;
	double* m_mass;
	double* m_pot;

public:


	REBO(int maxcount);
	virtual ~REBO();

	void SetListOrtho(int count);
		
	void SetListNonortho(mat33d Am, CELLDIV_INFO* info);
	void Force(int flag);

	void EJSum(double dt);

	double GetPotentialEnergy(int count);
	void GetFr(double* a_Fr);
	void GetFrv3d(vec3d* a_Fr_v);
	void GetFrm33d(mat33d* a_Fr_m);
	
	void SetParticlePointers(AtomArray* atoms);

	void GetBondingPointers(int* bcnt, REBO_BONDING** bonding);
	void GetREBOPointers(REBO_PARTICLE** prp);

};


#endif    // __rebojf_h__
