
#include "sdkdx11.h"

/////////////////////////////////////////////////////////////////////
// DirectX 11
D3D_DRIVER_TYPE         g_driverType = D3D_DRIVER_TYPE_NULL;
D3D_FEATURE_LEVEL       g_featureLevel = D3D_FEATURE_LEVEL_11_0;
ID3D11Device*           g_pd3dDevice = NULL;
ID3D11DeviceContext*    g_pImmediateContext = NULL;
IDXGISwapChain*         g_pSwapChain = NULL;
ID3D11RenderTargetView* g_pRenderTargetView = NULL;
ID3D11Texture2D*          g_pDepthStencil = NULL;		// [x/XeV
ID3D11DepthStencilView*   g_pDepthStencilView = NULL;	// [x/XeVEr[


//--------------------------------------------------------------------------------------
// Create Direct3D device and swap chain
//--------------------------------------------------------------------------------------
HRESULT sdkdx11Initialize(HWND hWnd, int init_windowmode, DX11_CORE* dx11_core){
    
	HRESULT hr = S_OK;

    RECT rc;
    GetClientRect( hWnd, &rc );
    UINT width = rc.right - rc.left;
    UINT height = rc.bottom - rc.top;

    UINT createDeviceFlags = 0;
#ifdef _DEBUG
    createDeviceFlags |= D3D11_CREATE_DEVICE_DEBUG;
#endif

    D3D_DRIVER_TYPE driverTypes[] =
    {
        D3D_DRIVER_TYPE_HARDWARE,		//HAL
        D3D_DRIVER_TYPE_WARP,			//@\\tgEFA_O.
        D3D_DRIVER_TYPE_REFERENCE,		//reference
    };
    UINT numDriverTypes = ARRAYSIZE( driverTypes );

    D3D_FEATURE_LEVEL featureLevels[] =
    {
        D3D_FEATURE_LEVEL_11_0,
        D3D_FEATURE_LEVEL_10_1,
        D3D_FEATURE_LEVEL_10_0,
    };
	UINT numFeatureLevels = ARRAYSIZE( featureLevels );

    DXGI_SWAP_CHAIN_DESC sd;
    ZeroMemory( &sd, sizeof( sd ) );
    sd.BufferCount = 2;				//3D vision ׂ̈ɂ2ݒ.
    sd.BufferDesc.Width = width;
    sd.BufferDesc.Height = height;
    sd.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
    sd.BufferDesc.RefreshRate.Numerator = 60;
    sd.BufferDesc.RefreshRate.Denominator = 1;
    sd.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
    sd.OutputWindow = hWnd;
    sd.SampleDesc.Count = 1;
    sd.SampleDesc.Quality = 0;
	if(init_windowmode){
		sd.Windowed = TRUE;	//TRUE(window mode), FALSE(fullscreen)
	}else{
		sd.Windowed = FALSE;	//TRUE(window mode), FALSE(fullscreen)
	}

    for( UINT driverTypeIndex = 0; driverTypeIndex < numDriverTypes; driverTypeIndex++ )
    {
        g_driverType = driverTypes[driverTypeIndex];
        hr = D3D11CreateDeviceAndSwapChain( NULL, g_driverType, NULL, createDeviceFlags, featureLevels, numFeatureLevels,
                                            D3D11_SDK_VERSION, &sd, &g_pSwapChain, &g_pd3dDevice, &g_featureLevel, &g_pImmediateContext );
        if( SUCCEEDED( hr ) )
            break;
    }
    if( FAILED( hr ) )
        return hr;

    // Create a render target view
    ID3D11Texture2D* pBackBuffer = NULL;
    hr = g_pSwapChain->GetBuffer( 0, __uuidof( ID3D11Texture2D ), ( LPVOID* )&pBackBuffer );
    if( FAILED( hr ) )
        return hr;

	//BackBufferp̑(View)쐬.
    hr = g_pd3dDevice->CreateRenderTargetView( pBackBuffer, NULL, &g_pRenderTargetView );
    if( FAILED( hr ) )
        return hr;
	
	//obNEobt@̏擾(for depthBuffer)
	D3D11_TEXTURE2D_DESC descBackBuffer;
	pBackBuffer->GetDesc(&descBackBuffer);

	//gȂ̂Ń[X.
    pBackBuffer->Release();


	//obNobt@o͐Ɏw
    g_pImmediateContext->OMSetRenderTargets( 1, &g_pRenderTargetView, NULL );

	//depth_buffer(as 2D texture)̍쐬.
	D3D11_TEXTURE2D_DESC descDepth;
	descDepth.Width     = descBackBuffer.Width;   // 
	descDepth.Height    = descBackBuffer.Height;  // 
	descDepth.MipLevels = 1;       // ~bv}bvEx
	descDepth.ArraySize = 1;       // zTCY
	descDepth.Format    = DXGI_FORMAT_D32_FLOAT;  // tH[}bg([x̂)
	descDepth.SampleDesc.Count   = descBackBuffer.SampleDesc.Count;  // }`TvO̐ݒ
	descDepth.SampleDesc.Quality = descBackBuffer.SampleDesc.Quality;  // }`TvO̕i
	descDepth.Usage          = D3D11_USAGE_DEFAULT;      // ftHggp@
	descDepth.BindFlags      = D3D11_BIND_DEPTH_STENCIL; // [x/XeVƂĎgp
	descDepth.CPUAccessFlags = 0;   // CPU̓ANZXȂ
	descDepth.MiscFlags      = 0;   // ̑̐ݒȂ
	hr = g_pd3dDevice->CreateTexture2D(
			&descDepth,         // 쐬2DeNX`̐ݒ
			NULL,               // 
			&g_pDepthStencil);  // 쐬eNX`󂯎ϐ
	if (FAILED(hr))
		return hr;

	// [x/XeV r[̍쐬
	D3D11_DEPTH_STENCIL_VIEW_DESC descDSV;
	descDSV.Format             = descDepth.Format;            // r[̃tH[}bg
	descDSV.ViewDimension      = D3D11_DSV_DIMENSION_TEXTURE2D;	//}`TvOȂD3D10_DSV_DIMENSION_TEXTURE2DMS
	descDSV.Flags              = 0;		//DX11
	descDSV.Texture2D.MipSlice = 0;		//}`TvOȂTexture2DMS
	hr = g_pd3dDevice->CreateDepthStencilView(
			g_pDepthStencil,       // [x/XeVEr[eNX`
			&descDSV,              // [x/XeVEr[̐ݒ
			&g_pDepthStencilView); // 쐬r[󂯎ϐ
	if (FAILED(hr))
		return hr;

	//fvXobt@̊֘At.
    g_pImmediateContext->OMSetRenderTargets( 1, &g_pRenderTargetView, g_pDepthStencilView );


    // Setup the viewport
    D3D11_VIEWPORT vp;
    vp.Width = (FLOAT)width;
    vp.Height = (FLOAT)height;
    vp.MinDepth = 0.0f;
    vp.MaxDepth = 1.0f;
    vp.TopLeftX = 0;
    vp.TopLeftY = 0;
    g_pImmediateContext->RSSetViewports( 1, &vp );


	//󂯓n//
	dx11_core->pd3dDevice = g_pd3dDevice;
	dx11_core->pd3dDC = g_pImmediateContext;
	dx11_core->pSwapChain = g_pSwapChain;
	dx11_core->pRenderTargetView = g_pRenderTargetView;
	dx11_core->pDepthStencilView = g_pDepthStencilView;



    return S_OK;
}

//--------------------------------------------------------------------------------------
// Clean up the objects we've created
//--------------------------------------------------------------------------------------
void sdkdx11Terminate(){

	BOOL fullscreen;
	g_pSwapChain->GetFullscreenState(&fullscreen, NULL);
	if( fullscreen ){ g_pSwapChain->SetFullscreenState(FALSE, NULL);	}

    if( g_pImmediateContext ) g_pImmediateContext->ClearState();
	
	if( g_pDepthStencilView ) g_pDepthStencilView->Release();
	if( g_pDepthStencil ) g_pDepthStencil->Release();

    if( g_pRenderTargetView ) g_pRenderTargetView->Release();
    if( g_pSwapChain ) {
	
		g_pSwapChain->Release();
	}
    if( g_pImmediateContext ) g_pImmediateContext->Release();
    if( g_pd3dDevice ) g_pd3dDevice->Release();
}
