
#ifndef __vec4_h__
#define __vec4_h__


#pragma once

#include <math.h>
#include <string.h>
#include "vec3.h"

	class vec4f;

//double================================
	class vec4d{
	public:
		double x;
		double y;
		double z;
		double w;

		inline vec4d& Set(const double& d1, const double& d2, const double& d3, const double& d4){
			x = d1;
			y = d2;
			z = d3;
			w = d4;
			return *this;
		};

		inline vec4d& operator +=(const vec4d& v){
			x += v.x;
			y += v.y;
			z += v.z;
			w += v.w;
			return *this;
		}
		
		inline vec4d operator -(){
			vec4d v2;
			v2.x = - x;
			v2.y = - y;
			v2.z = - z;
			v2.w = - w;
			return v2;
		}

		inline vec4d& operator -=(const vec4d& v){
			x -= v.x;
			y -= v.y;
			z -= v.z;
			w -= v.w;
			return *this;
		}
		
		inline vec4d& operator *=(const double& d){
			x *= d;
			y *= d;
			z *= d;
			w *= d;
			return *this;
		}
		
		inline vec4d& operator /=(const double& d){
			double di = 1.0 / d;
			x *= di;
			y *= di;
			z *= di;
			w *= di;
			return *this;
		}
		
		inline void Clear(){
			memset(&x, 0, 4 * sizeof(double));
		}
		
		inline operator vec4f();

		
	};

	inline double operator *(const vec4d& v, const vec4d& v2){
		return (v.x * v2.x + v.y * v2.y + v.z * v2.z + v.w * v2.w);
	}
	
	inline vec4d operator +(const vec4d& v, const vec4d& v2){
		vec4d vr;
		vr.x = v.x + v2.x;
		vr.y = v.y + v2.y;
		vr.z = v.z + v2.z;
		vr.w = v.w + v2.w;
		return vr;
	}

	inline vec4d operator -(const vec4d& v, const vec4d& v2){
		vec4d vr;
		vr.x = v.x - v2.x;
		vr.y = v.y - v2.y;
		vr.z = v.z - v2.z;
		vr.w = v.w - v2.w;
		return vr;
	}

	inline vec4d operator *(const vec4d& v, const double& d){
		vec4d v2;
		v2.x = v.x * d;
		v2.y = v.y * d;
		v2.z = v.z * d;
		v2.w = v.w * d;
		return v2;
	}
	
	inline vec4d operator /(const vec4d& v, const double& d){
		double di = 1.0 / d;
		vec4d v2;
		v2.x = v.x * di;
		v2.y = v.y * di;
		v2.z = v.z * di;
		v2.w = v.w * di;
		return v2;
	}			

	



//float================================
	class vec4f{
	public:
		float x;
		float y;
		float z;
		float w;

		inline vec4f& Set(const float& f1, const float& f2, const float& f3, const float& f4){
			x = f1;
			y = f2;
			z = f3;
			w = f4;
			return *this;
		};

		inline vec4f& operator +=(const vec4f& v){
			x += v.x;
			y += v.y;
			z += v.z;
			w += v.w;
			return *this;
		}

		inline vec4f operator -(){
			vec4f v2;
			v2.x = - x;
			v2.y = - y;
			v2.z = - z;
			v2.w = - w;
			return v2;
		}

		inline vec4f& operator -=(const vec4f& v){
			x -= v.x;
			y -= v.y;
			z -= v.z;
			w -= v.w;
			return *this;
		}
		
		inline vec4f& operator *=(const float& f){
			x *= f;
			y *= f;
			z *= f;
			w *= f;
			return *this;
		}
		
		inline vec4f& operator /=(const float& f){
			float fi = 1.0f / f;
			x *= fi;
			y *= fi;
			z *= fi;
			w *= fi;
			return *this;
		}
	
		inline void Clear(){
			memset(&x, 0, 4 * sizeof(float));
		}

	
		inline operator vec4d();

	};

	inline vec4f operator *(const vec4f& v, const float& f){
		vec4f v2;
		v2.x = v.x * f;
		v2.y = v.y * f;
		v2.z = v.z * f;
		v2.w = v.w * f;
		return v2;
	}
	
	inline vec4f operator /(const vec4f& v, const float& f){
		float fi = 1.0f / f;
		vec4f v2;
		v2.x = v.x * fi;
		v2.y = v.y * fi;
		v2.z = v.z * fi;
		v2.w = v.w * fi;
		return v2;
	}			
	
	inline float operator *(const vec4f& v, const vec4f& v2){
		return (v.x * v2.x + v.y * v2.y + v.z * v2.z + v.w * v2.w);
	}
	
	inline vec4f operator +(const vec4f& v, const vec4f& v2){
		vec4f vr;
		vr.x = v.x + v2.x;
		vr.y = v.y + v2.y;
		vr.z = v.z + v2.z;
		vr.w = v.w + v2.w;
		return vr;
	}

	inline vec4f operator -(const vec4f& v, const vec4f& v2){
		vec4f vr;
		vr.x = v.x - v2.x;
		vr.y = v.y - v2.y;
		vr.z = v.z - v2.z;
		vr.w = v.w - v2.w;
		return vr;
	}
		

//cast================================
	inline vec4d::operator vec4f(){
		vec4f vf;
		vf.x = (float)x;
		vf.y = (float)y;
		vf.z = (float)z;
		vf.w = (float)w;
		return vf;
	}
	
	inline vec4f::operator vec4d(){
		vec4d vd;
		vd.x = (double)x;
		vd.y = (double)y;
		vd.z = (double)z;
		vd.w = (double)w;
		return vd;
	}
	

#endif // __vec4_h__